@@ -1,6 +1,5 @@
 /*
  * Copyright (C) 2010 NVIDIA Corporation
- * Copyright (C) 2011 Google, Inc.
  *
  * This software is licensed under the terms of the GNU General Public
  * License version 2, as published by the Free Software Foundation, and
@@ -22,7 +21,6 @@
 #include <mach/pinmux.h>
 #include <mach/pinmux-t2.h>
 
-#include <asm/mach-types.h>
 #include "gpio-names.h"
 #include "board-seaboard.h"
 
@@ -206,22 +204,23 @@ static struct tegra_gpio_table common_gpio_table[] = {
 	{ .gpio = TEGRA_GPIO_USB3,		.enable = true },
 	{ .gpio = TEGRA_GPIO_NCT1008_THERM2_IRQ,.enable = true },
 	{ .gpio = TEGRA_GPIO_WLAN_POWER,	.enable = true },
-	{ .gpio = TEGRA_GPIO_BACKLIGHT,		.enable = true },
-	{ .gpio = TEGRA_GPIO_BACKLIGHT_VDD,	.enable = true },
-	{ .gpio = TEGRA_GPIO_LVDS_SHUTDOWN,	.enable = true },
-	{ .gpio = TEGRA_GPIO_EN_VDD_PNL,	.enable = true },
 	{ .gpio = TEGRA_GPIO_AC_ONLINE,		.enable = true },
 	{ .gpio = TEGRA_GPIO_HP_DET,		.enable = true },
-	{ .gpio = TEGRA_GPIO_CYTP_INT,		.enable = true },
 	{ .gpio = TEGRA_GPIO_MXT_IRQ,		.enable = true },
 	{ .gpio = TEGRA_GPIO_HDMI_ENB,		.enable = true },
 	{ .gpio = TEGRA_GPIO_MPU3050_IRQ,	.enable = true },
 	{ .gpio = TEGRA_GPIO_DISABLE_CHARGER,	.enable = true },
-	{ .gpio = TEGRA_GPIO_BT_RESET,		.enable = true },
-	{ .gpio = TEGRA_GPIO_HDMI_HPD,		.enable = true },
 	{ .gpio = TEGRA_GPIO_RECOVERY_SWITCH,	.enable = true },
 	{ .gpio = TEGRA_GPIO_DEV_SWITCH,	.enable = true },
 	{ .gpio = TEGRA_GPIO_WP_STATUS,		.enable = true },
+	{ .gpio = TEGRA_GPIO_BACKLIGHT,		.enable = true },
+	{ .gpio = TEGRA_GPIO_LVDS_SHUTDOWN,	.enable = true },
+	{ .gpio = TEGRA_GPIO_BACKLIGHT_VDD,	.enable = true },
+	{ .gpio = TEGRA_GPIO_EN_VDD_PNL,	.enable = true },
+	{ .gpio = TEGRA_GPIO_BT_RESET,		.enable = true },
+	{ .gpio = TEGRA_GPIO_BT_SHUTDOWN,	.enable = true },
+	{ .gpio = TEGRA_GPIO_HDMI_HPD,		.enable = true },
+	{ .gpio = TEGRA_GPIO_CYTP_INT,		.enable = true },
 };
 
 static struct tegra_gpio_table seaboard_gpio_table[] = {
@@ -233,50 +232,25 @@ static struct tegra_gpio_table asymptote_gpio_table[] = {
 	{ .gpio = ASYMPTOTE_GPIO_TSL2563_IRQ,	.enable = true },
 };
 
-
-static void __init sound_codec_gpio_init(int gpio, const char *name)
+void __init seaboard_common_pinmux_init(void)
 {
 	/*
 	 * PINGROUP_SPIC contains two pins:
 	 * + PX2, DISABLE_CHRGR (output)
-	 * + PX3, wm8903 codec IRQ (input)
-	 * + PX3, max98095 codec IRQ (input)
+	 * + PX3, WM8903 codec IRQ (input)
 	 *
-	 * The pinmux module can only configure TRISTATE vs. NORMAL on
-	 * a per-group rather than per-pin basis. The group must be
-	 * NORMAL since at least one pin is an output.
-	 *
-	 * However, we must ensure that the codec IRQ is never driven,
-	 * since the codec itself is driving it, and we don't want
-	 * multiple drivers.
-	 *
-	 *To ensure this, configure PX3 as a GPIO here, and set is as
-	 * an input, before the pinmux table is written, which is when
-	 * the pins will be un-tristated.
+	 * The pinmux module can only configure TRISTATE vs. NORMAL on a
+	 * per-group rather than per-pin basis. The group must be NORMAL
+	 * since at least one pin is an output. However, we must ensure that
+	 * the WM8903 IRQ is never driven, since the WM8903 itself is driving
+	 * it, and we don't want multiple drivers. To ensure this, configure
+	 * PX3 as a GPIO here, and set is as an input, before the pinmux table
+	 * is written, which is when the pins will be un-tristated.
 	 */
-	tegra_gpio_enable(gpio);
-	gpio_request(gpio, name);
-	gpio_direction_input(gpio);
-}
+	tegra_gpio_enable(TEGRA_GPIO_CDC_IRQ);
+	gpio_request(TEGRA_GPIO_CDC_IRQ, "wm8903");
+	gpio_direction_input(TEGRA_GPIO_CDC_IRQ);
 
-static void __init max98095_gpio_init(void)
-{
-	BUG_ON(!machine_is_arthur());
-	sound_codec_gpio_init(TEGRA_GPIO_MAX98095_IRQ, "max98095");
-}
-
-static void __init wm8903_gpio_init(void)
-{
-	BUG_ON(!machine_is_seaboard() &&
-	       !machine_is_kaen()     &&
-	       !machine_is_aebl()     &&
-	       !machine_is_asymptote() &&
-	       !machine_is_ventana());
-	sound_codec_gpio_init(TEGRA_GPIO_WM8903_IRQ, "wm8903");
-}
-
-static void __init seaboard_common_pinmux_init(void)
-{
 	/* Ensure the reset line stays high. */
 	gpio_request(TEGRA_GPIO_RESET, "reset");
 	gpio_direction_output(TEGRA_GPIO_RESET, 1);
@@ -290,6 +264,12 @@ static void __init seaboard_common_pinmux_init(void)
 	tegra_gpio_config(common_gpio_table, ARRAY_SIZE(common_gpio_table));
 }
 
+void __init seaboard_pinmux_init(void)
+{
+	seaboard_common_pinmux_init();
+	tegra_gpio_config(seaboard_gpio_table, ARRAY_SIZE(seaboard_gpio_table));
+}
+
 static void __init update_pinmux(struct tegra_pingroup_config *newtbl, int size)
 {
 	int i, j;
@@ -326,14 +306,6 @@ void __init fixup_pinmux_for_26Mhz(void)
 				  ARRAY_SIZE(pingroup_CDEV2_MUX_OSC));
 }
 
-void __init seaboard_pinmux_init(void)
-{
-	wm8903_gpio_init();
-	seaboard_common_pinmux_init();
-	tegra_gpio_config(seaboard_gpio_table,
-			  ARRAY_SIZE(seaboard_gpio_table));
-}
-
 void __init kaen_pinmux_init(void)
 {
 	/* TODO: Refactor this entire board revision specialization when
@@ -344,15 +316,13 @@ void __init kaen_pinmux_init(void)
 	int ram_id;
 	u32 reg;
 
-	wm8903_gpio_init();
-	seaboard_common_pinmux_init();
+	seaboard_pinmux_init();
 
 	/*
-	 * For Kaen DVT2 (and not DVT1) there is a single pinmux
-	 * register change to be made which accomodates a 26Mhz
-	 * reference oscillator.  If DVT2, then overwrite the value
-	 * that was just written in the call to
-	 * seaboard_common_pinmux_init().
+	 * For Kaen DVT2 (and not DVT1) there is a single pinmux register
+	 * change to be made which accomodates a 26Mhz reference oscillator.
+	 * If DVT2, then overwrite the value that was just written in the call
+	 * to seaboard_pinmux_init().
 	 *
 	 * Detect that 26Mhz oscillator is present for Kaen based on ram_id.
 	 *   ram_id = {0, 1} for DVT2, 26Mhz oscillator
@@ -367,24 +337,16 @@ void __init kaen_pinmux_init(void)
 
 void __init aebl_pinmux_init(void)
 {
-	wm8903_gpio_init();
-	seaboard_common_pinmux_init();
+	seaboard_pinmux_init();
 	fixup_pinmux_for_26Mhz();
 }
 
-void __init arthur_pinmux_init(void)
-{
-	max98095_gpio_init();
-	seaboard_common_pinmux_init();
-}
-
 static __initdata struct tegra_pingroup_config asymptote_pinmux[] = {
 	{TEGRA_PINGROUP_UCB,  TEGRA_MUX_NONE,      TEGRA_PUPD_NORMAL, TEGRA_TRI_NORMAL},
 };
 
 void __init asymptote_pinmux_init(void)
 {
-	wm8903_gpio_init();
 	update_pinmux(asymptote_pinmux, ARRAY_SIZE(asymptote_pinmux));
 	seaboard_common_pinmux_init();
 	tegra_gpio_config(asymptote_gpio_table,
@@ -414,7 +376,6 @@ static __initdata struct tegra_pingroup_config ventana_pinmux[] = {
 
 void __init ventana_pinmux_init(void)
 {
-	wm8903_gpio_init();
 	update_pinmux(ventana_pinmux, ARRAY_SIZE(ventana_pinmux));
-	seaboard_common_pinmux_init();
+	seaboard_pinmux_init();
 }
